package asynchrony;

import java.util.List;
import java.util.concurrent.*;

/**
 *
 */
public class FutureTaskExecDemo {

	private ExecutorService executor;

	public FutureTaskExecDemo() {
		this.executor = Executors.newCachedThreadPool();
	}

	public static void main(String args[]) {
		FutureTaskExecDemo server = new FutureTaskExecDemo();
		System.out.println("Starting FutureTaskExecDemo");
		Future<?> future1 = startDemoThread(server, 45);
		Future<?> future2 = startDemoThread(server, 35);
		Future<?> future3 = startDemoThread(server, 20);
		Future<?> future4 = startDemoThread(server, 15);
		Future<?> future5 = startDemoThread(server, 5);

		try {
			future1.get();
			future2.get();
			future3.get();
			future4.get();
			future5.get();
		} catch (InterruptedException e) {
			e.printStackTrace();
		} catch (ExecutionException e) {
			e.printStackTrace();
		}
		server.shutdown();
	}

	public void shutdown() {
		// prevent this executor service to accept new tasks (previously submitted tasks are executed)
		this.executor.shutdown(); 
	}
	
	public List<Runnable> shutdownNow() {
		// attempt to stop all actively executing tasks 
		return this.executor.shutdownNow(); 
	}

	protected static Future<?> startDemoThread(final FutureTaskExecDemo server,
			final int n) {
		return server.executor.submit(new Runnable() {
			public void run() {
				Future<Integer> future;
				System.out.println("CALLING fibonacci(" + n + ")");
				future = server.service(n);
				System.out.println("GOT future(fibonacci(" + n + "))");
				try {
					int val = future.get(5000,TimeUnit.MILLISECONDS).intValue();
					System.out.println("GOT fibonacci("+n+") = " + val);
				} catch (InterruptedException e) {
				} catch (ExecutionException e) {
				} catch(TimeoutException e) {
					future.cancel(true);
					server.shutdownNow();
					System.err.println("error: operation timed out");
					System.exit(1);
				}
			}
		});
	}

	public Future<Integer> service(final int n) {
		Callable<Integer> task = new Callable<Integer>() {
			public Integer call() {
				return new Integer(fibonacci(n));
			}
		};
		// submit a callable to the executor service which returns a future
		return executor.submit(task);
	}

	public static int fibonacci(int n) {
		if (n < 2)
			return 1;
		else
			return fibonacci(n - 1) + fibonacci(n - 2);
	}

}